/********************************************************************
    file:       main.cpp
	
    purpose:    module initialization and cleanup,
                Main Message Handler
*********************************************************************/

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "bangCommands.h"
#include "lsAlbumListAPI.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Message Handling
//
int g_lsMessages[] = {
    LM_GETREVID,		// RevID handling for about box
    0
};

#define REV_BUFFER 64   // assumed minimum size of LiteStep's RevID buffer

const char szMessageHandlerClass[] = "lsAlbumList";


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// globals
//
HINSTANCE g_hInstance = NULL;
HWND g_hMainWnd = NULL;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// MessageHandlerProc
//
// Main Message Handler Procedure
//
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
        // LiteStep wants to know our version info
        case LM_GETREVID:
        {
            if (wParam == 0 || wParam == 1)
            {
                lstrcpyn((char*)lParam, "lsAlbumList 2.0 (mritche/ilmcuts)", REV_BUFFER);
                return strlen((char*)lParam);
            }
            
            // something above failed, make sure we don't return something invalid
            lParam = NULL;
        }
        break;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
// Module Initialization
//
int initModuleEx(IN HWND hParent, IN HINSTANCE hInstance, IN LPCSTR /* szLSPath */)
{
    WNDCLASS wc;

    ZeroMemory(&wc, sizeof(WNDCLASS));

	// Register class for Main Message Handler
    wc.style = CS_GLOBALCLASS;
    wc.lpfnWndProc = MessageHandlerProc;
    wc.hInstance = hInstance;
    wc.lpszClassName = szMessageHandlerClass;
    
    if (!RegisterClass(&wc))
    {
        return 1; // FIXME
    }
    
    g_hInstance = hInstance;
    
    g_hMainWnd = CreateWindowEx(
        WS_EX_NOPARENTNOTIFY | WS_EX_TOOLWINDOW,
        szMessageHandlerClass,
        NULL,
        WS_POPUP | WS_DISABLED,
        0, 0, 0, 0,
        IsWindow(hParent) ? hParent : GetLitestepWnd(),
        0,
        g_hInstance,
        0);
    
    if (!IsWindow(g_hMainWnd))
    {
        return 1;	// FIXME
    }

	PostMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) g_hMainWnd,
		(LPARAM) g_lsMessages);

    AddBangCommands();

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
// module cleanup
//
void quitModule(IN HINSTANCE /* hInstance */)
{
    RemoveBangCommands();
    
    // tell LS to remove us from the message list
    SendMessage(GetLitestepWnd(),
        LM_UNREGISTERMESSAGE,
        (WPARAM) g_hMainWnd,
        (LPARAM) g_lsMessages);

	DestroyWindow(g_hMainWnd);  // FIXME

	UnregisterClass(szMessageHandlerClass, g_hInstance); // FIXME
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point, supposedly used to get rid of CRT...
//
BOOL APIENTRY DllMain(IN HINSTANCE hInstance, IN DWORD dwReason, LPVOID /* pvReserved */)
{
	if(dwReason == DLL_PROCESS_ATTACH)
	{
		DisableThreadLibraryCalls(hInstance);
	}

	return TRUE;
}